import 'package:flutter/material.dart';
import 'package:unify_api/unify.dart';
import 'package:unify_api_example/common/index.dart';
import 'package:unify_api_example/utils/index.dart';

class CoreLeadsExample extends StatefulWidget {
  final UnifyClient client;

  const CoreLeadsExample({Key key, this.client}) : super(key: key);
  @override
  _CoreLeadsExampleState createState() => _CoreLeadsExampleState();
}

class _CoreLeadsExampleState extends State<CoreLeadsExample> {
  List<CoreLead> _items;

  @override
  void initState() {
    widget.client.listCoreLeads().then((items) {
      setState(() {
        _items = items;
      });
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("Lead Tracker"),
      ),
      body: Builder(
        builder: (BuildContext context) {
          if (_items == null) return Center(child: CircularProgressIndicator());
          if (_items.isEmpty)
            return Center(child: Text("No Lead Tracker Found"));
          return ListView.builder(
            itemBuilder: (BuildContext context, int index) {
              final i = _items[index];
              return ListTile(
                title: Text(i.displayName),
                subtitle: Text(i.email),
                trailing: JsonButton(json: i?.json),
                onTap: () {
                  Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) =>
                          CoreLeadDetailsExample(lead: i.getDetails(), client: widget.client),
                    ),
                  );
                },
              );
            },
            itemCount: _items?.length ?? 0,
          );
        },
      ),
    );
  }
}

class CoreLeadDetailsExample extends StatefulWidget {
  final Future<CoreLeadDetails> lead;
  final UnifyClient client;

  const CoreLeadDetailsExample({Key key, @required this.lead, this.client})
      : super(key: key);
  @override
  _CoreLeadDetailsExampleState createState() => _CoreLeadDetailsExampleState();
}

class _CoreLeadDetailsExampleState extends State<CoreLeadDetailsExample> {
  CoreLeadDetails _details;

  @override
  void initState() {
    widget.lead.then((details) {
      setState(() {
        _details = details;
      });
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("Lead Tracker Details"),
        actions: <Widget>[
          JsonButton(json: _details?.json),
        ],
      ),
      body: FutureBuilder<CoreLeadDetails>(
        initialData: _details,
        future: widget.lead,
        builder:
            (BuildContext context, AsyncSnapshot<CoreLeadDetails> snapshot) {
          if (snapshot?.data == null)
            return Center(child: CircularProgressIndicator());

          CoreLeadDetails details = snapshot.data;
          return ListView(
            children: <Widget>[
              ListTile(
                title: Text("Full Name"),
                subtitle: Text(details.fullName),
              ),
              ListTile(
                title: Text("Email"),
                subtitle: Text(details.email),
              ),
              ListTile(
                title: Text("Primary Phone"),
                subtitle: Text(details.primaryPhone.toString()),
              ),
              ListTile(
                title: Text("Secondary Phone"),
                subtitle: Text(details.secondaryPhone.toString()),
              ),
              ListTile(
                title: Text("Tasks"),
                subtitle: Text("Tap To View"),
                onTap: () {
                  var items = details?.tasks ?? [];
                  List<Widget> _children = [];
                  _children = items
                      .map((i) => ListTile(
                            title: Text(i.type),
                            subtitle: Text(i?.dateTime?.toString() ?? ""),
                            trailing: JsonButton(json: i?.json),
                          ))
                      .toList();
                  showList(context, _children);
                },
              ),
              ListTile(
                title: Text('Notes'),
                subtitle: Text('Tap To View'),
                onTap: () async {
                  var notes = await widget.client.listCoreLeadNotes(id: details.coreLeadId);
                  List<Widget> _children = [];
                  _children = notes.map((i) => ListTile(
                    title: Text(i.type),
                    subtitle: Text(i.note),
                    trailing: JsonButton(json: i?.json),
                  )).toList();
                  showList(context, _children);
                },
              ),
              ListTile(
                title: Text('Debug'),
                subtitle: Text('Test PhoneNumber bug'),
                onTap: () async {
                  try {
                     await widget.client.addCoreLead(
                    update: true,
                    updateID: details?.id,
                    borrower_First_Name: 'Devonnn',
                    next_Step: 'testing UnifyAPI',
                    next_Action_Date: '8/22/2020 12:00:00 AM',
                    initial_Appointment_Date: '8/22/2020 12:00:00 AM',
                    disposition: 'Pending',
                    appointment_Type: 'Phone',
                    hot_Lead: 'High',
                    assign_Lead_To: 'Manager, Test'
                  );
                  } catch (e) {               
                    debugPrint(e.toString());
                  }               
                },
              ),
              ListTile(
                title: Text('Add CoreLead'),
                subtitle: Text('test add "test, lead" coreLead'),
                onTap: () async {
                  try {
                    await widget.client.addCoreLead(
                      borrower_First_Name: 'Example',
                      borrower_Last_Name: 'test'
                    );
                  } catch (e) {
                    debugPrint(e.toString());
                  }
                },
              )
            ],
          );
        },
      ),
    );
  }
}
